module rate_adaption_fifo_rd_ctrl (
	ENABLE,
		
	CLK,
	CLK_ENA,
	RESET_N,
	
	//To MII/GMII Interface
	MII_ENA,
	MII_DATA,
	MII_ERR,
	
	//From FIFO	
	FIFO_RD_ADDR,
	FIFO_RD_ENA,
	FIFO_RD_DATA,
	
	//Control
	PPM_VALID,
	PPM_COUNT
);

parameter	MII_DATA_WIDTH = 8;
parameter	FIFO_ADDR_WIDTH = 6;
parameter	FIFO_DATA_WIDTH = 16;
parameter	PPM_COUNT_SIZE = 6; 

parameter	RD_DLY_THRESHOLD = 16;

//FSM Parameter
parameter	FSM_WIDTH 	= 3;
parameter	IDLE		= 3'b001;
parameter	DIS_RD_ENA	= 3'b010;
parameter	HOLD_N_WAIT	= 3'b100;

input							ENABLE;
input							CLK;
input							CLK_ENA;
input							RESET_N;

//To MII/GMII
output							MII_ENA;
output	[MII_DATA_WIDTH-1:0]	MII_DATA;
output							MII_ERR;

//To FIFO
output	[FIFO_ADDR_WIDTH-1:0]	FIFO_RD_ADDR;
output							FIFO_RD_ENA;
input	[FIFO_DATA_WIDTH-1:0]	FIFO_RD_DATA;

//Control
input							PPM_VALID;
input	[PPM_COUNT_SIZE-1:0]	PPM_COUNT;

//register
reg		[FIFO_ADDR_WIDTH-1:0]	fifo_addr_reg;
reg								fifo_rd_ena_reg;

reg		[PPM_COUNT_SIZE-1:0]	ppm_count_reg;
reg		[PPM_COUNT_SIZE-1:0]	ppm_count_dec;

reg		[FSM_WIDTH-1:0] 		state;
reg		[FSM_WIDTH-1:0] 		next;

reg								MII_ENA;
reg		[MII_DATA_WIDTH-1:0]	MII_DATA;
reg								MII_ERR;

reg		[FIFO_ADDR_WIDTH-1:0]	read_dly_count;

wire								mii_ena_sig;
wire		[MII_DATA_WIDTH-1:0]	mii_data_sig;
wire								mii_err_sig;

assign 	FIFO_RD_ADDR = fifo_addr_reg;
assign	FIFO_RD_ENA = fifo_rd_ena_reg;  	

assign	mii_ena_sig = FIFO_RD_DATA[MII_DATA_WIDTH+1];
assign	mii_data_sig = FIFO_RD_DATA[MII_DATA_WIDTH-1:0];
assign	mii_err_sig = FIFO_RD_DATA[MII_DATA_WIDTH];   


////////////////////////////////////////////////////////////////////////////////////
//MII Interface
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N)
		MII_ENA <= 1'b0;
	else if (CLK_ENA)
		MII_ENA <= mii_ena_sig;
		
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N)
		MII_ERR <= 1'b0;
	else if (CLK_ENA)
		MII_ERR <= mii_err_sig;
		
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N)
		MII_DATA <= {MII_DATA_WIDTH{1'b0}};
	else if (CLK_ENA)
		MII_DATA <= mii_data_sig;
		
////////////////////////////////////////////////////////////////////////////////////
//FIFO Interface
//FIFO Address
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		fifo_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
	end
	else if (CLK_ENA) begin
		if (ENABLE && (read_dly_count == RD_DLY_THRESHOLD)) begin
			if (next == DIS_RD_ENA)
				fifo_addr_reg <= fifo_addr_reg;
			else
				fifo_addr_reg <= fifo_addr_reg + 1'b1;
		end
		else begin
			fifo_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
		end
	end

//FIFO Read Enable	
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		fifo_rd_ena_reg <= 1'b0;
	end
	else if (CLK_ENA) begin
		if (ENABLE && (read_dly_count == RD_DLY_THRESHOLD)) begin
			if (next == DIS_RD_ENA)
				fifo_rd_ena_reg <= 1'b0;
			else
				fifo_rd_ena_reg <= 1'b1;
		end
		else begin
			fifo_rd_ena_reg <= 1'b0;
		end	
	end
	else begin
		fifo_rd_ena_reg <= 1'b0;
	end
	
//Read Delay Counter
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		read_dly_count <= {FIFO_ADDR_WIDTH{1'b0}};
	end
	else if (CLK_ENA) begin
		if (ENABLE && (read_dly_count == RD_DLY_THRESHOLD))
			read_dly_count <= read_dly_count;
		else if (ENABLE)
			read_dly_count <= read_dly_count + 1'b1; 		
		else
			read_dly_count <= {FIFO_ADDR_WIDTH{1'b0}}; 
	end
	
///////////////////////////////////////////////////////////	
// PPM Control register					
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		ppm_count_reg <= {PPM_COUNT_SIZE{1'b0}};
	end
	else if (CLK_ENA) begin
		if (PPM_VALID)
			//ppm_count_reg <= ppm_count_reg + PPM_COUNT;	//original	
			ppm_count_reg <= PPM_COUNT;	
		else if (next == IDLE)
			ppm_count_reg <= {PPM_COUNT_SIZE{1'b0}};	
	end
	

always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		ppm_count_dec <= {PPM_COUNT_SIZE{1'b0}};
	end
	else if (CLK_ENA) begin
		if (next == IDLE)
			ppm_count_dec <= {PPM_COUNT_SIZE{1'b0}};
		else if (next == DIS_RD_ENA)
			ppm_count_dec <= ppm_count_dec + 1'b1;	
	end

////////////////////////////////////////////////////////////////////////////////////
//FSM
//State Machine
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		state <= IDLE;
	end
	else if (CLK_ENA) begin
		if (!ENABLE)
			state <= IDLE;
		else
			state <= next; 
	end

always @(*)
begin
	next = IDLE;
	
	case (state)
		IDLE: begin
			if (!mii_ena_sig && (ppm_count_reg != {PPM_COUNT_SIZE{1'b0}}))
				next = DIS_RD_ENA;
			else if (PPM_VALID && |PPM_COUNT && mii_ena_sig)
				next = HOLD_N_WAIT; 
			else
				next = IDLE;
		end
		
		DIS_RD_ENA: begin
			if (ppm_count_reg == ppm_count_dec)
				next = IDLE;
			else if (mii_ena_sig && (ppm_count_reg != ppm_count_dec))
				next = HOLD_N_WAIT;
			else
				next = DIS_RD_ENA; 
		end
	
		HOLD_N_WAIT: begin
			if (!mii_ena_sig)
				next = DIS_RD_ENA;
			else if (PPM_VALID && !(|PPM_COUNT) && mii_ena_sig)
				next = IDLE;	
			else
				next = HOLD_N_WAIT;
		end
		
		default: begin
			next = IDLE;
		end

	endcase
end

////////////////////////////////////////////////////////////////////////////////////

endmodule