module rate_adaption_fifo_wr_ctrl (
	ENABLE,
		
	CLK,
	CLK_ENA,
	RESET_N,
	
	//From MII/GMII Interface
	MII_ENA,
	MII_DATA,
	MII_ERR,
	
	//To FIFO	
	FIFO_WR_ADDR,
	FIFO_WR_ENA,
	FIFO_WR_DATA,
	
	//Control
	PPM_VALID,
	PPM_COUNT
);

parameter	MII_DATA_WIDTH = 8;
parameter	FIFO_ADDR_WIDTH = 6;
parameter	FIFO_DATA_WIDTH = 16;
parameter	PPM_COUNT_SIZE = 6; 

//FSM Parameter
parameter	FSM_WIDTH 	= 3;
parameter	IDLE		= 3'b001;
parameter	DIS_WR_ENA	= 3'b010;
parameter	HOLD_N_WAIT	= 3'b100;

input							ENABLE;
input							CLK;
input							CLK_ENA;
input							RESET_N;

//From MII/GMII
input							MII_ENA;
input	[MII_DATA_WIDTH-1:0]	MII_DATA;
input							MII_ERR;

//To FIFO
output	[FIFO_ADDR_WIDTH-1:0]	FIFO_WR_ADDR;
output							FIFO_WR_ENA;
output	[FIFO_DATA_WIDTH-1:0]	FIFO_WR_DATA;

//Control
input							PPM_VALID;
input	[PPM_COUNT_SIZE-1:0]	PPM_COUNT;

//register
reg		[FIFO_ADDR_WIDTH-1:0]	fifo_addr_reg;
reg		[FIFO_DATA_WIDTH-1:0]	fifo_data_reg;
reg								fifo_wr_ena_reg;

reg		[PPM_COUNT_SIZE-1:0]	ppm_count_reg;
reg		[PPM_COUNT_SIZE-1:0]	ppm_count_dec;

reg		[FSM_WIDTH-1:0] 		state;
reg		[FSM_WIDTH-1:0] 		next;

//wire
wire	[MII_DATA_WIDTH+1:0]	fifo_data_bus;

////////////////////////////////////////////////////////////////////////////////////
//FIFO Write Interface
assign	fifo_data_bus = {MII_ENA, MII_ERR, MII_DATA}; 
assign	FIFO_WR_ENA = fifo_wr_ena_reg;
assign	FIFO_WR_ADDR = fifo_addr_reg;
assign	FIFO_WR_DATA = fifo_data_reg;   

always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		fifo_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
	end
	else if (CLK_ENA) begin
		if (ENABLE) begin
			if (next == DIS_WR_ENA)
				fifo_addr_reg <= fifo_addr_reg;
			else
				fifo_addr_reg <= fifo_addr_reg + 1'b1;
			end
		else begin
			fifo_addr_reg <= {FIFO_ADDR_WIDTH{1'b0}};
		end
	end

always @(posedge CLK or negedge RESET_N)
	if (!RESET_N)
		fifo_data_reg <= {FIFO_DATA_WIDTH{1'b0}};
	else if (CLK_ENA && ENABLE) 
		fifo_data_reg <= {{FIFO_DATA_WIDTH-MII_DATA_WIDTH-2{1'b0}}, fifo_data_bus}; 

always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		fifo_wr_ena_reg <= 1'b0;
	end
	else if (CLK_ENA) begin
		if (ENABLE) begin
			if (next == DIS_WR_ENA)
				fifo_wr_ena_reg <= 1'b0;
			else
				fifo_wr_ena_reg <= 1'b1;
		end
		else begin
			fifo_wr_ena_reg <= 1'b0;
		end
	end
	else begin
		fifo_wr_ena_reg <= 1'b0;
	end
		
///////////////////////////////////////////////////////////
// PPM Control register					
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		ppm_count_reg <= {PPM_COUNT_SIZE{1'b0}};
	end
	else if (CLK_ENA) begin
		if (PPM_VALID)
			//ppm_count_reg <= ppm_count_reg + PPM_COUNT;	//original
			ppm_count_reg <= PPM_COUNT;	
		else if (next == IDLE)
			ppm_count_reg <= {PPM_COUNT_SIZE{1'b0}};	
	end
	

always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		ppm_count_dec <= {PPM_COUNT_SIZE{1'b0}};
	end
	else if (CLK_ENA) begin
		if (next == IDLE)
			ppm_count_dec <= {PPM_COUNT_SIZE{1'b0}};
		else if (next == DIS_WR_ENA)
			ppm_count_dec <= ppm_count_dec + 1'b1;
	end
	
////////////////////////////////////////////////////////////////////////////////////
//FSM
//State Machine
always @(posedge CLK or negedge RESET_N)
	if (!RESET_N) begin
		state <= IDLE;
	end
	else if (CLK_ENA) begin
		if (!ENABLE)
			state <= IDLE;
		else
			state <= next; 
	end

always @(*)
begin
	next = IDLE;
	
	case (state)
		IDLE: begin
			if (!MII_ENA && (ppm_count_reg != {PPM_COUNT_SIZE{1'b0}}))
				next = DIS_WR_ENA;
			else if (PPM_VALID && |PPM_COUNT && MII_ENA)
				next = HOLD_N_WAIT; 
			else
				next = IDLE;
		end
		
		DIS_WR_ENA: begin
			if (ppm_count_reg == ppm_count_dec)
				next = IDLE;
			else if (MII_ENA && (ppm_count_reg != ppm_count_dec))
				next = HOLD_N_WAIT;	
			else
				next = DIS_WR_ENA; 
		end
	
		HOLD_N_WAIT: begin
			if (!MII_ENA)
				next = DIS_WR_ENA;
			else if (PPM_VALID && !(|PPM_COUNT) && MII_ENA)
				next = IDLE;	
			else
				next = HOLD_N_WAIT;
		end
		
		default: begin
			next = IDLE;
		end

	endcase
end

////////////////////////////////////////////////////////////////////////////////////


endmodule